@@ -44,14 +44,12 @@ class ArtifactFormat(SubDir):
 
     @cachedmethod(lambda self: self._is_reserved_cache)
     def _is_reserved_file(self, path, *, resolve=False):
-        internal = self._internal_path()
         digests = self._digest_path()
         dirty = self._dirty_path()
         if resolve:
-            internal = internal.resolve()
             digests = digests.resolve()
             dirty = dirty.resolve()
-        return path in (internal, digests, dirty)
+        return path in (digests, dirty)
 
     def list_dir(self, relpath: str) -> Tuple[List[str], List[str]]:
         path = self.get_filepath(relpath)
@@ -82,6 +80,13 @@ class ArtifactFormat(SubDir):
     def get_digests(self) -> Dict[str, str]:
         try:
             return json.loads(self._digest_path().read_text())
+        except FileNotFoundError:
+            if self.name != "container":
+                return {}
+            try:
+                return {f: f"sha256:{os.path.basename(f)}" for f in self.get_files()}
+            except OSError:
+                return {}
         except (OSError, ValueError):
             return {}
 
@@ -119,19 +124,6 @@ class ArtifactFormat(SubDir):
         digests[relpath] = digest
         self._digest_path().write_text(json.dumps(digests))
 
-    def _internal_path(self) -> Path:
-        return self._path / ".internal"
-
-    def is_internal(self) -> bool:
-        return self._internal_path().is_file()
-
-    def set_internal(self, internal: bool):
-        path = self._internal_path()
-        if internal:
-            path.touch()
-        elif path.is_file():
-            path.unlink()
-
     def _dirty_path(self) -> Path:
         return self._path / ".dirty"
 
@@ -141,7 +133,7 @@ class ArtifactFormat(SubDir):
     def set_dirty(self, dirty: bool):
         path = self._dirty_path()
         if dirty:
-            path.parent.mkdir(mode=0o755, parents=True, exist_ok=True)
+            self.create()
             path.touch()
         elif path.is_file():
             path.unlink()
