@@ -66,11 +66,11 @@ class Tag(SubDir):
     def is_released(self) -> bool:
         return self._released_path().is_file()
 
-    def set_released(self, released: bool):
+    def set_released(self, released: bool, semaphore: asyncio.Semaphore):
         if not self._path.is_dir():
             raise FileNotFoundError()
         loop = asyncio.get_running_loop()
-        task = loop.create_task(self.do_release(released))
+        task = loop.create_task(self.do_release(released, semaphore))
         task.add_done_callback(self.done_cb)
 
     def done_cb(self, task):
@@ -99,7 +99,7 @@ class Tag(SubDir):
             headers={"User-Agent": self.USER_AGENT},
         )
 
-    async def do_release(self, released: bool):
+    async def do_release(self, released: bool, semaphore: asyncio.Semaphore):
         if self.PUBLISH_URL and self.PUBLISH_AUTH:
             self._publish_status_path().write_text("in progress\n")
             async with self._publish_session() as sess:
@@ -110,7 +110,7 @@ class Tag(SubDir):
                         self.name,
                         self.PUBLISH_URL,
                     )
-                    await self._publish(sess)
+                    await self._publish(sess, semaphore)
                 else:
                     LOG.info(
                         "deleting tag %s/%s from %s",
@@ -118,53 +118,78 @@ class Tag(SubDir):
                         self.name,
                         self.PUBLISH_URL,
                     )
-                    await sess.delete(self.url(), params={"force": "true"})
+                    async with semaphore:
+                        await sess.delete(self.url(), params={"force": "true"})
+                    p = self._publish_status_path()
+                    if p.is_file():
+                        p.unlink()
         path = self._released_path()
         if released:
             path.touch()
         elif path.is_file():
             path.unlink()
 
-    async def _publish(self, sess: aiohttp.ClientSession):
+    async def _publish(self, sess: aiohttp.ClientSession, semaphore: asyncio.Semaphore):
+        loop = asyncio.get_running_loop()
         for job in self.get_jobs():
+            if job.is_internal():
+                continue
+            job_url = job.url()
+            # in case job is already present on the server, delete it before re-uploading it
+            await sess.delete(job_url, params={"force": "true"}, raise_for_status=False)
             self._publish_status_path().write_text(f"uploading {job.name}\n")
+            tasks = []
             for fmt in job.get_formats():
-                await self._publish_fmt(fmt, sess)
+                tasks.append(loop.create_task(self._publish_fmt(fmt, sess, semaphore)))
+            await asyncio.gather(*tasks)
             metadata = job.get_metadata()
             del metadata["name"]
             del metadata["locked"]
-            job_url = job.url()
             LOG.debug("publishing job metadata %s", job_url)
-            await sess.patch(job_url, json={"job": metadata})
+            async with semaphore:
+                await sess.patch(job_url, json={"job": metadata})
         self._publish_status_path().write_text(f"published to {self.PUBLISH_URL}\n")
 
-    async def _publish_fmt(self, fmt: ArtifactFormat, sess: aiohttp.ClientSession):
+    async def _publish_fmt(
+        self,
+        fmt: ArtifactFormat,
+        sess: aiohttp.ClientSession,
+        semaphore: asyncio.Semaphore,
+    ):
         fmt_url = fmt.url()
+        loop = asyncio.get_running_loop()
 
-        for file, digest in fmt.get_digests().items():
+        async def _publish_file(file, digest):
             file_url = fmt_url + file
             headers = {"Digest": digest}
 
-            resp = await sess.head(file_url, headers=headers, raise_for_status=False)
+            async with semaphore:
+                resp = await sess.head(
+                    file_url, headers=headers, raise_for_status=False
+                )
             if resp.status == 200:
                 LOG.debug("publishing file %s (deduplicated)", file_url)
                 # file digest already present on the server, do not upload
                 # the data again
                 headers["X-Dlrepo-Link"] = digest
-                await sess.put(file_url, data=None, headers=headers)
+                async with semaphore:
+                    await sess.put(file_url, data=None, headers=headers)
 
             else:
                 LOG.debug("publishing file %s", file_url)
                 # file digest not on server, proceed with upload
-                with open(fmt.path() / file, "rb") as f:
-                    await sess.put(file_url, data=f, headers=headers)
+                async with semaphore:
+                    with open(fmt.path() / file, "rb") as f:
+                        await sess.put(file_url, data=f, headers=headers)
 
-        if fmt.is_internal():
-            LOG.debug("publishing internal format %s", fmt_url)
-            await sess.put(fmt_url, json={"artifact_format": {"internal": True}})
+        tasks = []
+        for file, digest in fmt.get_digests().items():
+            tasks.append(loop.create_task(_publish_file(file, digest)))
+        await asyncio.gather(*tasks)
 
         # clear the dirty flag
-        await sess.patch(fmt_url)
+        async with semaphore:
+            await sess.patch(fmt_url)
 
     def _locked_path(self) -> Path:
         return self._path / ".locked"
