@@ -48,9 +48,12 @@ class ArtifactView(BaseView):
                 if match["tag"] == "latest":
                     raise web.HTTPMethodNotAllowed(self.request.method, ["GET"])
                 if match["format"] == "container":
-                    raise web.HTTPBadRequest(
-                        reason="Uploading container images must be done with docker push"
-                    )
+                    agent = self.request.headers.get("User-Agent", "")
+                    # allow uploading directly between dlrepo servers
+                    if not agent.startswith("dlrepo-server/"):
+                        raise web.HTTPBadRequest(
+                            reason="Uploading container images must be done with docker push"
+                        )
                 if job.is_locked():
                     raise web.HTTPBadRequest(
                         reason="Cannot upload files in locked jobs"
@@ -128,7 +131,7 @@ class ArtifactView(BaseView):
         headers = {}
         if digest:
             headers["Digest"] = digest
-        return web.FileResponse(filepath, headers=headers)
+        return self.file_response(filepath, headers=headers)
 
     async def put(self):
         fmt = self._get_format(create=True)
