@@ -2,6 +2,8 @@
 # Copyright (c) 2021 Robin Jarry
 # SPDX-License-Identifier: BSD-3-Clause
 
+import asyncio
+
 from aiohttp import web
 import aiohttp_jinja2
 
@@ -85,6 +87,7 @@ class BranchView(BaseView):
                     "timestamp": t.timestamp,
                     "released": t.is_released(),
                     "locked": t.is_locked(),
+                    "publish_status": t.publish_status(),
                 }
             )
         if "html" in self.request.headers.get("Accept", "json"):
@@ -114,9 +117,11 @@ class BranchView(BaseView):
         """
         Delete a branch and all its contents.
         """
+        loop = asyncio.get_running_loop()
         branch = self._get_branch()
         try:
-            branch.delete(force="force" in self.request.query)
+            force = "force" in self.request.query
+            await loop.run_in_executor(None, lambda: branch.delete(force=force))
             return web.Response()
         except OSError as e:
             raise web.HTTPBadRequest(reason=str(e)) from e
