@@ -13,7 +13,12 @@ from .util import BaseView
 # --------------------------------------------------------------------------------------
 class ContainerView(BaseView):
     def response(self, body=None, path=None, status=HTTPStatus.OK, headers=None):
-        resp = super().response(body, path, status, headers)
+        if isinstance(body, (dict, list)):
+            resp = web.json_response(body, status=status, headers=headers)
+        elif path is not None:
+            resp = self.file_response(path, status=status, headers=headers)
+        else:
+            resp = web.Response(body=body, status=status, headers=headers)
         resp.headers["Docker-Distribution-Api-Version"] = "registry/2.0"
         return resp
 
@@ -61,9 +66,9 @@ class ManifestReadOnlyView(ContainerView):
     @classmethod
     def urls(cls):
         yield "/v2/{branch}/{job}/manifests/{reference}"
-        yield "/v2/{user}/{branch}/{job}/manifests/{reference}"
+        yield "/v2/u/{user}/{branch}/{job}/manifests/{reference}"
         yield "/v2/{product}/{variant}/{product_branch}/manifests/{reference}"
-        yield "/v2/{user}/{product}/{variant}/{product_branch}/manifests/{reference}"
+        yield "/v2/u/{user}/{product}/{variant}/{product_branch}/manifests/{reference}"
 
     def get_manifest(self):
         match = self.request.match_info
@@ -117,7 +122,7 @@ class ManifestView(ManifestReadOnlyView):
     @classmethod
     def urls(cls):
         yield "/v2/{branch}/{job}/manifests/{reference}"
-        yield "/v2/{user}/{branch}/{job}/manifests/{reference}"
+        yield "/v2/u/{user}/{branch}/{job}/manifests/{reference}"
 
     async def put(self):
         match = self.request.match_info
@@ -160,9 +165,9 @@ class TagsListView(ContainerView):
     @classmethod
     def urls(cls):
         yield "/v2/{branch}/{job}/tags/list"
-        yield "/v2/{user}/{branch}/{job}/tags/list"
+        yield "/v2/u/{user}/{branch}/{job}/tags/list"
         yield "/v2/{product}/{variant}/{product_branch}/tags/list"
-        yield "/v2/{user}/{product}/{variant}/{product_branch}/tags/list"
+        yield "/v2/u/{user}/{product}/{variant}/{product_branch}/tags/list"
 
     async def get(self):
         match = self.request.match_info
@@ -195,7 +200,7 @@ class NewBlobUploadView(ContainerView):
     @classmethod
     def urls(cls):
         yield "/v2/{branch}/{job}/blobs/uploads/"
-        yield "/v2/{user}/{branch}/{job}/blobs/uploads/"
+        yield "/v2/u/{user}/{branch}/{job}/blobs/uploads/"
 
     async def post(self):
         uuid = self.repo().next_upload()
@@ -214,7 +219,7 @@ class BlobsUploadsView(ContainerView):
     @classmethod
     def urls(cls):
         yield "/v2/{branch}/{job}/blobs/uploads/{uuid}"
-        yield "/v2/{user}/{branch}/{job}/blobs/uploads/{uuid}"
+        yield "/v2/u/{user}/{branch}/{job}/blobs/uploads/{uuid}"
 
     async def patch(self):
         uuid = self.request.match_info["uuid"]
@@ -270,9 +275,9 @@ class BlobsView(ContainerView):
     @classmethod
     def urls(cls):
         yield "/v2/{branch}/{job}/blobs/{digest}"
-        yield "/v2/{user}/{branch}/{job}/blobs/{digest}"
+        yield "/v2/u/{user}/{branch}/{job}/blobs/{digest}"
         yield "/v2/{product}/{variant}/{product_branch}/blobs/{digest}"
-        yield "/v2/{user}/{product}/{variant}/{product_branch}/blobs/{digest}"
+        yield "/v2/u/{user}/{product}/{variant}/{product_branch}/blobs/{digest}"
 
     def _blob_path(self):
         digest = self.request.match_info["digest"]
