@@ -2,6 +2,8 @@
 # Copyright (c) 2021 Robin Jarry
 # SPDX-License-Identifier: BSD-3-Clause
 
+import asyncio
+
 from aiohttp import web
 import aiohttp_jinja2
 
@@ -65,11 +67,27 @@ class JobView(JobArchiveView):
             raise web.HTTPFound(job.url())
         html = "html" in self.request.headers.get("Accept", "json")
         data = {"job": job.get_metadata()}
+        data["job"]["internal"] = job.is_internal()
+        data["job"]["timestamp"] = job.timestamp
         formats = []
         for f in job.get_formats():
-            if self.access_granted(f.url()):
+            fmt_url = f.url()
+            if self.access_granted(fmt_url):
                 if html:
-                    formats.append({"name": f.name, "internal": f.is_internal()})
+                    digests = f.get_digests()
+                    deb = rpm = False
+                    if "repodata/repomd.xml" in digests:
+                        rpm = True
+                    elif "Release" in digests:
+                        deb = True
+                    formats.append(
+                        {
+                            "name": f.name,
+                            "rpm": rpm,
+                            "deb": deb,
+                            "url": fmt_url,
+                        }
+                    )
                 else:
                     formats.append(f.name)
         data["job"]["artifact_formats"] = formats
@@ -78,16 +96,26 @@ class JobView(JobArchiveView):
         return web.json_response(data)
 
     async def put(self):
+        job = self._get_job()
         try:
-            locked = (await self.json_body())["job"]["locked"]
-            if not isinstance(locked, bool):
+            data = (await self.json_body())["job"]
+            internal = data.get("internal")
+            if internal is not None and not isinstance(internal, bool):
+                raise TypeError()
+            locked = data.get("locked")
+            if locked is not None and not isinstance(locked, bool):
                 raise TypeError()
         except (TypeError, KeyError) as e:
             raise web.HTTPBadRequest(reason="Invalid parameters") from e
+
         try:
-            self._get_job().set_locked(locked)
+            if internal is not None:
+                job.set_internal(internal)
+            if locked is not None:
+                job.set_locked(locked)
         except FileNotFoundError as e:
             raise web.HTTPNotFound() from e
+
         return web.Response()
 
     async def patch(self):
@@ -110,9 +138,10 @@ class JobView(JobArchiveView):
         """
         Delete a job and all its contents.
         """
+        loop = asyncio.get_running_loop()
         try:
             job = self._get_job()
-            job.delete()
+            await loop.run_in_executor(None, job.delete)
         except FileNotFoundError as e:
             raise web.HTTPNotFound() from e
         except OSError as e:
